package com.cultural.heritage.controller.book;


import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.BookConstant;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonStringRequest;
import com.cultural.heritage.model.dto.book.BookingDateAddRequest;
import com.cultural.heritage.model.dto.timeinterval.TimeIntervalAddRequest;
import com.cultural.heritage.model.entity.BookingDate;
import com.cultural.heritage.model.entity.TimeInterval;
import com.cultural.heritage.model.vo.book.BookingDateVO;
import com.cultural.heritage.model.vo.timeinterval.TimeIntervalVO;
import com.cultural.heritage.service.book.BookingDateService;
import com.cultural.heritage.service.book.TimeIntervalService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/book")
@Slf4j
@Tag(name = "写真预约日期管理模块")
public class BookingDateController {


    @Resource
    private BookingDateService bookingDateService;


    @Resource
    private TimeIntervalService timeIntervalService;



    /**
     * Web端管理员添加写真预约日期和时间段
     * @param bookingDateAddRequestList 预约日期列表
     * @return 是否添加成功
     */
    @PostMapping("/add")
    @Operation(summary = "Web端管理员添加写真预约日期和时间段", description = "参数：预约日期列表，权限：管理员(admin, boss)，方法名：addBookingDate")
    @Transactional(rollbackFor = Exception.class)
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> addBookingDate(@RequestBody List<BookingDateAddRequest> bookingDateAddRequestList) {
        if (bookingDateAddRequestList == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 添加写真预约日期
        List<BookingDate> bookingDateList = bookingDateAddRequestList.stream().map(bookingDateAddRequest -> {
            BookingDate bookingDate = new BookingDate();
            BeanUtils.copyProperties(bookingDateAddRequest, bookingDate);
            bookingDateService.validBookingDate(bookingDate, false);
            return bookingDate;
        }).toList();
        boolean result = bookingDateService.saveBatch(bookingDateList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);

        // 添加写真预约时间段
        List<TimeInterval> timeIntervals = new ArrayList<>();
        for (int i = 0; i < bookingDateAddRequestList.size(); i++) {
            BookingDate bookingDate = bookingDateList.get(i);
            Long dateId = bookingDate.getId();
            BookingDateAddRequest bookingDateAddRequest = bookingDateAddRequestList.get(i);
            List<TimeIntervalAddRequest> timeIntervalAddRequestList = bookingDateAddRequest.getTimeIntervalAddRequestList();
            List<TimeInterval> timeIntervalList = timeIntervalAddRequestList.stream().map(timeIntervalAddRequest -> {
                TimeInterval timeInterval = new TimeInterval();
                BeanUtils.copyProperties(timeIntervalAddRequest, timeInterval);
                timeInterval.setBookingDateId(dateId);
                timeIntervalService.validTimeInterval(timeInterval, false);
                return timeInterval;
            }).toList();
            timeIntervals.addAll(timeIntervalList);
        }
        boolean success = timeIntervalService.saveBatch(timeIntervals);
        ThrowUtils.throwIf(!success, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }


    /**
     * Web端管理员根据预约类型查询预约时间表
     * @param commonStringRequest  预约类型
     * @return 预约时间表
     */
    @PostMapping("/list/type")
    @Operation(summary = "Web端管理员根据预约类型查询预约时间表", description = "参数：预约类型，权限：管理员(admin, boss)，方法名：listBookingDateByType")
    @Transactional(rollbackFor = Exception.class)
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<List<BookingDateVO>> listBookingDateByType(@RequestBody CommonStringRequest commonStringRequest) {
        if (commonStringRequest == null || StringUtils.isBlank(commonStringRequest.getType())) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        String type = commonStringRequest.getType();
        if (!type.equals(BookConstant.RENT_CLOTHES) && !type.equals(BookConstant.OWN_CLOTHES)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "类型参数错误");
        }
        QueryWrapper<BookingDate> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("type", type);
        List<BookingDate> bookingDateList = bookingDateService.list(queryWrapper);
        List<TimeInterval> timeIntervalList = timeIntervalService.list();
        // 存储写真预约时间段的map集合(键:预约日期id, 值:预约时间段列表)
        Map<Long, List<TimeIntervalVO>> timeMap = new HashMap<>();
        // 处理预约时间段，将预约时间段存入map集合
        for (TimeInterval timeInterval : timeIntervalList) {
            TimeIntervalVO timeIntervalVO = new TimeIntervalVO();
            BeanUtils.copyProperties(timeInterval, timeIntervalVO);
            Long bookingDateId = timeInterval.getBookingDateId();
            List<TimeIntervalVO> timeIntervalVOS = timeMap.get(bookingDateId);
            if (timeIntervalVOS == null) {
                timeIntervalVOS = new ArrayList<>();
            }
            timeIntervalVOS.add(timeIntervalVO);
            timeMap.put(bookingDateId, timeIntervalVOS);
        }
        // 处理预约日期，填充当前预约日期对应的预约时间段
        List<BookingDateVO> bookingDateVOList = new ArrayList<>();
        for (BookingDate bookingDate : bookingDateList) {
            BookingDateVO bookingDateVO = new BookingDateVO();
            BeanUtils.copyProperties(bookingDate, bookingDateVO);
            Long dateId = bookingDate.getId();
            List<TimeIntervalVO> timeIntervalVOS = timeMap.get(dateId);
            bookingDateVO.setTimeIntervalVOList(timeIntervalVOS);
            bookingDateVOList.add(bookingDateVO);
        }

        // 将预约日期从小到大排序
        bookingDateVOList.sort((book1, book2) -> {
            DateTime date1 = DateUtil.parse(book1.getSpecificDate(), "yyyy-MM-dd");
            DateTime date2 = DateUtil.parse(book2.getSpecificDate(), "yyyy-MM-dd");
            return date1.compareTo(date2);
        });

        return ResultUtils.success(bookingDateVOList);
    }



//    @PostMapping("/update")
//    @Operation(summary = "Web端管理员根据预约类型更新预约时间表", description = "参数：预约类型，权限：管理员(admin, boss)，方法名：updateBookingDate")
//    @Transactional(rollbackFor = Exception.class)
//    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
//    public BaseResponse<Boolean> updateBookingDate(@RequestBody CommonStringRequest commonStringRequest) {
//        if (commonStringRequest == null || StringUtils.isBlank(commonStringRequest.getType())) {
//            throw new BusinessException(ErrorCode.PARAMS_ERROR);
//        }
//        String type = commonStringRequest.getType();
//        if (!type.equals(BookConstant.RENT_CLOTHES) && !type.equals(BookConstant.OWN_CLOTHES)) {
//            throw new BusinessException(ErrorCode.PARAMS_ERROR, "类型参数错误");
//        }
//        QueryWrapper<BookingDate> queryWrapper = new QueryWrapper<>();
//        queryWrapper.eq("type", type);
//        return null;
//    }


}
