package com.cultural.heritage.controller.good;


import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonBatchRequest;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.appointment.AppointmentDateAddRequest;
import com.cultural.heritage.model.dto.good.GoodAddRequest;
import com.cultural.heritage.model.dto.good.GoodQueryRequest;
import com.cultural.heritage.model.dto.good.GoodUpdateRequest;
import com.cultural.heritage.model.dto.good.service.ServiceGoodAddRequest;
import com.cultural.heritage.model.dto.good.service.ServiceGoodQueryRequest;
import com.cultural.heritage.model.dto.good.service.ServiceGoodSingleUpdateRequest;
import com.cultural.heritage.model.dto.good.service.ServiceGoodUpdateRequest;
import com.cultural.heritage.model.dto.timeperiod.TimePeriodAddRequest;
import com.cultural.heritage.model.entity.AppointmentDate;
import com.cultural.heritage.model.entity.CartRecord;
import com.cultural.heritage.model.entity.Good;
import com.cultural.heritage.model.entity.TimePeriod;
import com.cultural.heritage.model.vo.appointment.AppointmentDateTimePeriodVO;
import com.cultural.heritage.model.vo.appointment.AppointmentDateVO;
import com.cultural.heritage.model.vo.good.GoodPageVO;
import com.cultural.heritage.model.vo.good.ServiceGoodCardVO;
import com.cultural.heritage.model.vo.good.ServiceGoodVO;
import com.cultural.heritage.model.vo.timeperiod.TimePeriodVO;
import com.cultural.heritage.service.good.AppointmentDateService;
import com.cultural.heritage.service.good.CartRecordService;
import com.cultural.heritage.service.good.GoodService;
import com.cultural.heritage.service.good.TimePeriodService;
import com.cultural.heritage.service.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.*;


/**
 * 商品接口
 */
@RestController
@RequestMapping("/goods")
@Slf4j
@Tag(name = "商品管理模块")
public class GoodController {


    @Resource
    private GoodService goodService;


    @Resource
    private UserService userService;


    @Resource
    private AppointmentDateService appointmentDateService;


    @Resource
    private TimePeriodService timePeriodService;


    @Resource
    private CartRecordService cartRecordService;




    /**
     * Web端管理员添加常规类商品
     * @param goodAddRequest  商品添加请求体
     * @return 是否添加成功
     */
    @PostMapping("/add")
    @Operation(summary = "Web端管理员添加常规类商品", description = "参数：商品添加请求体，权限：管理员(admin, boss)，方法名：addGood")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> addGood(@RequestBody GoodAddRequest goodAddRequest) {
        if (goodAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Good good = new Good();
        BeanUtils.copyProperties(goodAddRequest, good);
        // 校验
        goodService.validGood(good, false);
        boolean save = goodService.save(good);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true, "商品插入成功");
    }




    /**
     * Web端管理员添加服务类商品
     * @param serviceGoodAddRequest  服务类商品添加请求体
     * @return 是否添加成功
     */
    @PostMapping("/add/service")
    @Transactional(rollbackFor = Exception.class)
    @Operation(summary = "Web端管理员添加服务类商品", description = "参数：服务类商品添加请求体，权限：管理员(admin, boss)，方法名：addServiceGood")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> addServiceGood(@RequestBody ServiceGoodAddRequest serviceGoodAddRequest) {
        if (serviceGoodAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 向商品表插入服务类商品的基本信息
        Good good = new Good();
        BeanUtils.copyProperties(serviceGoodAddRequest, good);
        good.setType("服务类");
        good.setIsGoodType(0);
        good.setInventory(1);
        good.setFestivalName("无");
        // 校验
        goodService.validGood(good, false);
        boolean save = goodService.save(good);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR);

        // 添加当前商品的预约日期
        List<AppointmentDateAddRequest> appointmentDateAddRequestList = serviceGoodAddRequest.getAppointmentDateAddRequestList();
        List<AppointmentDate> appointmentDateList = appointmentDateAddRequestList.stream().map(appointmentDateAddRequest -> {
            AppointmentDate appointmentDate = new AppointmentDate();
            BeanUtils.copyProperties(appointmentDateAddRequest, appointmentDate);
            appointmentDate.setGoodId(good.getId());
            // 校验
            appointmentDateService.validAppointmentDate(appointmentDate, false);
            return appointmentDate;
        }).toList();
        boolean isSaveBatch = appointmentDateService.saveBatch(appointmentDateList);
        ThrowUtils.throwIf(!isSaveBatch, ErrorCode.OPERATION_ERROR);

        // 添加当前商品的预约时间段
        List<TimePeriod> timePeriods = new ArrayList<>();
        for (int i = 0; i < appointmentDateAddRequestList.size(); i++) {
            AppointmentDate appointmentDate = appointmentDateList.get(i);
            AppointmentDateAddRequest appointmentDateAddRequest = appointmentDateAddRequestList.get(i);
            Long appointmentDateId = appointmentDate.getId();
            List<TimePeriodAddRequest> timePeriodAddRequestList = appointmentDateAddRequest.getTimePeriodAddRequestList();
            List<TimePeriod> timePeriodList = timePeriodAddRequestList.stream().map(timePeriodAddRequest -> {
                TimePeriod timePeriod = new TimePeriod();
                BeanUtils.copyProperties(timePeriodAddRequest, timePeriod);
                timePeriod.setAppointmentDateId(appointmentDateId);
                // 校验
                timePeriodService.validTimePeriod(timePeriod, false);
                return timePeriod;
            }).toList();
            timePeriods.addAll(timePeriodList);
        }
        boolean result = timePeriodService.saveBatch(timePeriods);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }



    /**
     * Web端管理员删除常规类商品
     * @param deleteRequest   商品删除请求体
     * @return  是否删除成功
     */
    @PostMapping("/delete")
    @Operation(summary = "Web端管理员删除常规类商品", description = "参数：商品删除请求体，权限：管理员(admin, boss)，方法名：deleteGood")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> deleteGood(@RequestBody CommonRequest deleteRequest) {
        if (deleteRequest == null || deleteRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = deleteRequest.getId();
        boolean result = goodService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }



    /**
     * Web端管理员删除服务类商品
     * @param deleteRequest     服务类商品删除请求体
     * @return  是否删除成功
     */
    @PostMapping("/delete/service")
    @Operation(summary = "Web端管理员删除服务类商品", description = "参数: 商品删除请求体，权限：管理员(admin, boss)，方法名：deleteServiceGood")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<Boolean> deleteServiceGood(@RequestBody CommonRequest deleteRequest) {
        if (deleteRequest == null || deleteRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = deleteRequest.getId();

        //删除预约时间段表中与该商品关联的记录
        QueryWrapper<AppointmentDate> dateQueryWrapper = new QueryWrapper<>();
        dateQueryWrapper.eq("goodId", id);
        List<AppointmentDate> appointmentDateList = appointmentDateService.list(dateQueryWrapper);
        List<Long> ids = appointmentDateList.stream().map(AppointmentDate::getId).toList();
        QueryWrapper<TimePeriod> timePeriodQueryWrapper = new QueryWrapper<>();
        timePeriodQueryWrapper.in("appointmentDateId", ids);
        boolean remove = timePeriodService.remove(timePeriodQueryWrapper);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR, "服务类商品预约时间段删除失败");

        // 删除预约日期表中与该商品关联的记录
        boolean isSuccess = appointmentDateService.remove(dateQueryWrapper);
        ThrowUtils.throwIf(!isSuccess, ErrorCode.OPERATION_ERROR, "服务类商品预约日期删除失败");

        // 删除商品表中的服务类商品
        boolean result = goodService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "服务类商品删除失败");
        return ResultUtils.success(true);
    }





    /**
     * Web端用户批量删除服务类商品
     * @param ids   服务类商品id列表
     * @return   是否删除成功
     */
    @PostMapping("/delBatch/service")
    @Operation(summary = "Web端用户批量删除服务类商品", description = "服务类商品id列表，权限：管理员(admin, boss)，方法名：delBatchServiceGoods")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<Boolean> delBatchServiceGoods(@RequestBody List<Long> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        //删除预约时间段表中与该商品关联的记录
        QueryWrapper<AppointmentDate> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("goodId", ids);
        List<AppointmentDate> appointmentDateList = appointmentDateService.list(queryWrapper);
        List<Long> appointmentDateIds = appointmentDateList.stream().map(AppointmentDate::getId).toList();
        QueryWrapper<TimePeriod> timePeriodQueryWrapper = new QueryWrapper<>();
        timePeriodQueryWrapper.in("appointmentDateId", appointmentDateIds);
        boolean success = timePeriodService.remove(timePeriodQueryWrapper);
        ThrowUtils.throwIf(!success, ErrorCode.OPERATION_ERROR, "服务类商品预约时间段删除失败");

        // 删除预约日期表中与该商品关联的记录
        boolean remove = appointmentDateService.remove(queryWrapper);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR, "服务类商品预约日期删除失败");

        // 删除商品表中的服务类商品
        boolean result = goodService.removeBatchByIds(ids);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "服务类商品删除失败");

        return ResultUtils.success(true);

    }



    /**
     * Web端管理员更新常规类商品
     * @param goodUpdateRequest  商品更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "Web端管理员更新常规类商品", description = "参数：商品更新请求体，权限：管理员(admin, boss)，方法名：updateGoods")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateGoods(@RequestBody GoodUpdateRequest goodUpdateRequest) {
        if (goodUpdateRequest == null || goodUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        // 获取原来的商品价格
        Long sourceId = goodUpdateRequest.getId();
        Good resourceGood = goodService.getById(sourceId);
        BigDecimal resourcePrice = resourceGood.getPrice();
        Good good = new Good();
        BeanUtils.copyProperties(goodUpdateRequest, good);
        // 校验
        goodService.validGood(good, true);
        boolean result = goodService.updateById(good);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        // 如果更新了商品的价格，就需要更新购物车中的小计
        Long id = good.getId();
        BigDecimal targetPrice = good.getPrice();

        if (resourcePrice.compareTo(targetPrice) != 0) {
            // 更新购物车中的小计
            List<CartRecord> cartRecordList = goodService.updateCartGoodPrice(id, targetPrice);
            boolean update = cartRecordService.updateBatchById(cartRecordList);
            ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "购物车商品价格更新失败");
        }
        return ResultUtils.success(true);
    }




    /**
     * Web端管理员分页查询常规类商品
     * @param goodQueryRequest  查询商品请求体
     * @return  商品信息列表
     */
    @PostMapping("/list/page")
    @Operation(summary = "Web端管理员分页查询常规类商品", description = "参数：商品查询请求体，排序字段 sortField: price inventory id，权限：管理员(admin, boss)，方法名：listGoodByPage")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<GoodPageVO>> listGoodByPage(@RequestBody GoodQueryRequest goodQueryRequest) {
        long current = goodQueryRequest.getCurrent();
        long pageSize = goodQueryRequest.getPageSize();
        QueryWrapper<Good> goodQueryWrapper = goodService.getGoodQueryWrapper(goodQueryRequest, false);
        Page<Good> page = goodService.page(new Page<>(current, pageSize), goodQueryWrapper);
        List<Good> records = page.getRecords();
        List<GoodPageVO> goodPageVOS = records.stream().map(good -> {
            GoodPageVO goodPageVO = new GoodPageVO();
            BeanUtils.copyProperties(good, goodPageVO);
            return goodPageVO;
        }).toList();
        Page<GoodPageVO> goodPageVOPage = new Page<>();
        goodPageVOPage.setRecords(goodPageVOS);
        goodPageVOPage.setTotal(page.getTotal());
        goodPageVOPage.setSize(page.getSize());
        goodPageVOPage.setCurrent(page.getCurrent());
        goodPageVOPage.setPages(page.getPages());
        return ResultUtils.success(goodPageVOPage);
    }





    /**
     * Web端管理员批量删除常规类商品
     * @param commonDelBatchRequest 批量删除请求体
     * @return 是否批量删除成功
     */
    @PostMapping("/delBatch")
    @Operation(summary = "Web端管理员批量删除常规类商品", description = "参数：商品批量删除请求体，权限：管理员(admin, boss)，方法名：delBatchGoods")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> delBatchGoods(@RequestBody CommonBatchRequest commonDelBatchRequest) {
        List<Long> idList = commonDelBatchRequest.getIdList();
        boolean result = goodService.removeBatchByIds(idList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }





    /**
     * 小程序端用户根据id查询常规类商品
     * @param getByIdRequest  根据id查询请求体
     * @return  商品信息
     */
    @PostMapping("/getById")
    @Operation(summary = "小程序端用户根据id查询常规类商品", description = "参数：商品根据id查询请求体，权限：所有人, 方法名:getGoodById")
    public BaseResponse<GoodPageVO> getGoodById(@RequestBody CommonRequest getByIdRequest) {
        if (getByIdRequest == null || getByIdRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = getByIdRequest.getId();
        Good good = goodService.getById(id);
        ThrowUtils.throwIf(good == null, ErrorCode.NOT_FOUND_ERROR);
        GoodPageVO goodPageVO = new GoodPageVO();
        BeanUtils.copyProperties(good, goodPageVO);
        return ResultUtils.success(goodPageVO);
    }



    /**
     * Web端管理员分页查询服务类商品
     * @param serviceGoodQueryRequest  服务类商品查询请求体
     * @return  服务类商品列表
     */
    @PostMapping("/service/list/page")
    @Operation(summary = "Web端管理员分页查询服务类商品", description = "参数：商品查询请求体，权限：管理员(admin, boss)，方法名：listServiceGoodVOByPage")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<ServiceGoodVO>> listServiceGoodVOByPage(@RequestBody ServiceGoodQueryRequest serviceGoodQueryRequest) {
        if (serviceGoodQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = serviceGoodQueryRequest.getCurrent();
        long pageSize = serviceGoodQueryRequest.getPageSize();
        GoodQueryRequest goodQueryRequest = new GoodQueryRequest();
        BeanUtils.copyProperties(serviceGoodQueryRequest, goodQueryRequest);
        goodQueryRequest.setFestivalName("无");
        goodQueryRequest.setType("服务类");
        QueryWrapper<Good> goodQueryWrapper = goodService.getGoodQueryWrapper(goodQueryRequest, true);
        Page<Good> page = goodService.page(new Page<>(current, pageSize), goodQueryWrapper);
        List<Good> records = page.getRecords();

        // 获取联表(预约日期表, 预约时间段表)查询结果
        List<AppointmentDateTimePeriodVO> appointmentDateTimePeriodVOS = appointmentDateService.queryAppointmentDateDetail();
        List<AppointmentDateVO> appointmentDateVOList = new ArrayList<>();
        // 利用map集合封装预约日期和预约时间段
        Map<Long, List<TimePeriodVO>> timeMap = new HashMap<>();
        Map<Long, AppointmentDateVO> dateMap = new HashMap<>();
        for (AppointmentDateTimePeriodVO appointmentDateTimePeriodVO : appointmentDateTimePeriodVOS) {
            // 封装预约日期和预约时间段
            AppointmentDateVO appointmentDateVO = new AppointmentDateVO();
            TimePeriodVO timePeriodVO = new TimePeriodVO();
            BeanUtils.copyProperties(appointmentDateTimePeriodVO, appointmentDateVO);
            BeanUtils.copyProperties(appointmentDateTimePeriodVO, timePeriodVO);
            Long timePeriodId = appointmentDateTimePeriodVO.getTimePeriodId();
            timePeriodVO.setId(timePeriodId);

            Long id = appointmentDateTimePeriodVO.getId();
            List<TimePeriodVO> timePeriodVOList = timeMap.get(id);
            if (timePeriodVOList == null) {
                timePeriodVOList = new ArrayList<>();
                // 用map集合存储预约日期
                dateMap.put(id, appointmentDateVO);
            }
            timePeriodVOList.add(timePeriodVO);
            timeMap.put(id, timePeriodVOList);

        }
        // 封装预约日期集合
        Set<Long> dateIds = dateMap.keySet();
        for (Long dateId : dateIds) {
            AppointmentDateVO appointmentDateVO = dateMap.get(dateId);
            List<TimePeriodVO> timePeriodVOList = timeMap.get(dateId);
            appointmentDateVO.setTimePeriodVOList(timePeriodVOList);
            appointmentDateVOList.add(appointmentDateVO);
        }

        // 将预约日期从小到大排序
        appointmentDateVOList.sort((app1, app2) -> {
            DateTime date1 = DateUtil.parse(app1.getSpecificDate(), "yyyy-MM-dd");
            DateTime date2 = DateUtil.parse(app2.getSpecificDate(), "yyyy-MM-dd");
            return date1.compareTo(date2);
        });

        List<ServiceGoodVO> serviceGoodVOList = new ArrayList<>();
        for (Good good : records) {
            ServiceGoodVO serviceGoodVO = new ServiceGoodVO();
            BeanUtils.copyProperties(good, serviceGoodVO);
            Long goodVOId = serviceGoodVO.getId();
            List<AppointmentDateVO> dateVOList = appointmentDateVOList.stream()
                            .filter(appointmentDateVO -> Objects.equals(appointmentDateVO.getGoodId(), goodVOId)).toList();
            serviceGoodVO.setAppointmentDateVOList(dateVOList);
            serviceGoodVOList.add(serviceGoodVO);
        }

        Page<ServiceGoodVO> serviceGoodVOPage = new Page<>();
        serviceGoodVOPage.setRecords(serviceGoodVOList);
        serviceGoodVOPage.setPages(page.getPages());
        serviceGoodVOPage.setCurrent(page.getCurrent());
        serviceGoodVOPage.setSize(page.getSize());
        serviceGoodVOPage.setTotal(page.getTotal());
        return ResultUtils.success(serviceGoodVOPage);

    }




    /**
     * 小程序端用户根据id查询服务类商品
     * @param commonRequest 根据id查询请求体
     * @return 服务类商品信息
     */
    @PostMapping("/service/list/id")
    @Operation(summary = "小程序端用户根据id查询服务类商品", description = "参数：根据id查询请求体，权限：所有人，方法名：listServiceGoodVO")
    public BaseResponse<ServiceGoodVO> listServiceGoodVO(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        Long id = commonRequest.getId();
        Good good = goodService.getById(id);
        ThrowUtils.throwIf(good == null, ErrorCode.NOT_FOUND_ERROR, "商品不存在");
        ServiceGoodVO serviceGoodVO = new ServiceGoodVO();
        BeanUtils.copyProperties(good, serviceGoodVO);
        Long goodVOId = serviceGoodVO.getId();
        List<AppointmentDateTimePeriodVO> appointmentDateTimePeriodVOList = appointmentDateService.queryAppointmentDateDetailById(goodVOId);
        List<AppointmentDateVO> appointmentDateVOList = new ArrayList<>();
        // 封装预约日期和预约时间段
        Map<Long, List<TimePeriodVO>> timeMap = new HashMap<>();
        Map<Long, AppointmentDateVO> dateMap = new HashMap<>();
        for (AppointmentDateTimePeriodVO appointmentDateTimePeriodVO : appointmentDateTimePeriodVOList) {
            TimePeriodVO timePeriodVO = new TimePeriodVO();
            AppointmentDateVO appointmentDateVO = new AppointmentDateVO();
            BeanUtils.copyProperties(appointmentDateTimePeriodVO, timePeriodVO);
            BeanUtils.copyProperties(appointmentDateTimePeriodVO, appointmentDateVO);
            Long timePeriodId = appointmentDateTimePeriodVO.getTimePeriodId();
            timePeriodVO.setId(timePeriodId);

            Long appId = appointmentDateTimePeriodVO.getId();
            List<TimePeriodVO> timePeriodVOList = timeMap.get(appId);
            if (timePeriodVOList == null) {
                timePeriodVOList = new ArrayList<>();
                dateMap.put(appId, appointmentDateVO);
            }
            timePeriodVOList.add(timePeriodVO);
            timeMap.put(appId, timePeriodVOList);
        }

        Set<Long> dateIds = dateMap.keySet();
        for (Long dateId : dateIds) {
            AppointmentDateVO appointmentDateVO = dateMap.get(dateId);
            List<TimePeriodVO> timePeriodVOList = timeMap.get(dateId);
            appointmentDateVO.setTimePeriodVOList(timePeriodVOList);
            appointmentDateVOList.add(appointmentDateVO);
        }

        // 将预约日期从小到大排序
        appointmentDateVOList.sort((app1, app2) -> {
            DateTime date1 = DateUtil.parse(app1.getSpecificDate(), "yyyy-MM-dd");
            DateTime date2 = DateUtil.parse(app2.getSpecificDate(), "yyyy-MM-dd");
            return date1.compareTo(date2);
        });

        serviceGoodVO.setAppointmentDateVOList(appointmentDateVOList);
        return ResultUtils.success(serviceGoodVO);
    }



    /**
     * 小程序端展示服务类商品卡片
     * @return  服务类商品卡片列表
     */
    @GetMapping("/service/list/card")
    @Operation(summary = "小程序端展示服务类商品卡片", description = "参数：无，权限：所有人，方法名：listServiceGoodCardVO")
    public BaseResponse<List<ServiceGoodCardVO>> listServiceGoodCardVO(HttpServletRequest request) {
        userService.getLoginUser(request);
        QueryWrapper<Good> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("isGoodType", 0);
        List<Good> goodList = goodService.list(queryWrapper);
        List<ServiceGoodCardVO> serviceGoodCardVOS = goodList.stream().map(good -> {
            ServiceGoodCardVO serviceGoodCardVO = new ServiceGoodCardVO();
            BeanUtils.copyProperties(good, serviceGoodCardVO);
            return serviceGoodCardVO;
        }).toList();
        return ResultUtils.success(serviceGoodCardVOS);
    }





    /**
     * Web端管理员更新服务类商品
     * @param serviceGoodUpdateRequest  服务类商品更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/service/update")
    @Operation(summary = "Web端管理员更新服务类商品", description = "参数：服务类商品更新请求体，权限：管理员(admin, boss)，方法名：updateServiceGoodById")
    @Transactional(rollbackFor = Exception.class)
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateServiceGoodById(@RequestBody ServiceGoodUpdateRequest serviceGoodUpdateRequest) {
        if (serviceGoodUpdateRequest == null || serviceGoodUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 更新服务类商品的基本信息
        Good good = new Good();
        BeanUtils.copyProperties(serviceGoodUpdateRequest, good);
        good.setType("服务类");
        good.setIsGoodType(0);
        good.setInventory(1);
        good.setFestivalName("无");
        // 校验
        goodService.validGood(good, true);
        boolean result = goodService.updateById(good);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "服务类商品信息更新失败");

        // 删除当前商品关联的所有日期和时间段
        CommonRequest commonRequest = new CommonRequest();
        commonRequest.setId(good.getId());
        deleteServiceGoodAppointment(commonRequest);
        // 为当前商品重新添加预约日期和时间段
        ServiceGoodAddRequest serviceGoodAddRequest = new ServiceGoodAddRequest();
        BeanUtils.copyProperties(serviceGoodUpdateRequest, serviceGoodAddRequest);
        dealServiceGoodAppointmentDate(serviceGoodAddRequest, good);
        return ResultUtils.success(true);
    }



    private void deleteServiceGoodAppointment(CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();

        //删除预约时间段表中与该商品关联的记录
        QueryWrapper<AppointmentDate> dateQueryWrapper = new QueryWrapper<>();
        dateQueryWrapper.eq("goodId", id);
        List<AppointmentDate> appointmentDateList = appointmentDateService.list(dateQueryWrapper);
        List<Long> ids = appointmentDateList.stream().map(AppointmentDate::getId).toList();
        QueryWrapper<TimePeriod> timePeriodQueryWrapper = new QueryWrapper<>();
        timePeriodQueryWrapper.in("appointmentDateId", ids);
        boolean remove = timePeriodService.remove(timePeriodQueryWrapper);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR, "服务类商品预约时间段删除失败");

        // 删除预约日期表中与该商品关联的记录
        boolean isSuccess = appointmentDateService.remove(dateQueryWrapper);
        ThrowUtils.throwIf(!isSuccess, ErrorCode.OPERATION_ERROR, "服务类商品预约日期删除失败");
    }


    private void dealServiceGoodAppointmentDate(ServiceGoodAddRequest serviceGoodAddRequest, Good good) {
        // 添加当前商品的预约日期
        List<AppointmentDateAddRequest> appointmentDateAddRequestList = serviceGoodAddRequest.getAppointmentDateAddRequestList();
        List<AppointmentDate> appointmentDateList = appointmentDateAddRequestList.stream().map(appointmentDateAddRequest -> {
            AppointmentDate appointmentDate = new AppointmentDate();
            BeanUtils.copyProperties(appointmentDateAddRequest, appointmentDate);
            appointmentDate.setGoodId(good.getId());
            // 校验
            appointmentDateService.validAppointmentDate(appointmentDate, false);
            return appointmentDate;
        }).toList();
        boolean isSaveBatch = appointmentDateService.saveBatch(appointmentDateList);
        ThrowUtils.throwIf(!isSaveBatch, ErrorCode.OPERATION_ERROR);

        // 添加当前商品的预约时间段
        List<TimePeriod> timePeriods = new ArrayList<>();
        for (int i = 0; i < appointmentDateAddRequestList.size(); i++) {
            AppointmentDate appointmentDate = appointmentDateList.get(i);
            AppointmentDateAddRequest appointmentDateAddRequest = appointmentDateAddRequestList.get(i);
            Long appointmentDateId = appointmentDate.getId();
            List<TimePeriodAddRequest> timePeriodAddRequestList = appointmentDateAddRequest.getTimePeriodAddRequestList();
            List<TimePeriod> timePeriodList = timePeriodAddRequestList.stream().map(timePeriodAddRequest -> {
                TimePeriod timePeriod = new TimePeriod();
                BeanUtils.copyProperties(timePeriodAddRequest, timePeriod);
                timePeriod.setAppointmentDateId(appointmentDateId);
                // 校验
                timePeriodService.validTimePeriod(timePeriod, false);
                return timePeriod;
            }).toList();
            timePeriods.addAll(timePeriodList);
        }
        boolean result = timePeriodService.saveBatch(timePeriods);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
    }



    /**
     * Web端管理员上(下)架服务类商品
     * @param commonRequest 优惠券上架状态更新请求体
     * @return 是否更新成功
     */
    @PostMapping("/service/shelves")
    @Operation(summary = "Web端管理员上(下)架服务类商品", description = "参数：优惠券上架状态更新请求体，权限：管理员(admin, boss)，方法名：updateServiceGoodShelvesStatus")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateServiceGoodShelvesStatus(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 获取当前服务类商品的上(下)架状态
        Long id = commonRequest.getId();
        Good good = goodService.getById(id);
        Integer status = good.getIsShelves() == 0 ? 1 : 0;
        UpdateWrapper<Good> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("isShelves", status);
        boolean update = goodService.update(updateWrapper);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "上架状态更新失败");
        return ResultUtils.success(true);
    }



    /**
     * Web端管理员单独更新服务类商品
     * @param serviceGoodSingleUpdateRequest  服务类商品更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/service/single/update")
    @Operation(summary = "Web端管理员单独更新服务类商品", description = "参数：服务类商品更新请求体，权限：管理员(admin, boss)，方法名：updateServiceGoodSingleById")
    @Transactional(rollbackFor = Exception.class)
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateServiceGoodSingleById(@RequestBody ServiceGoodSingleUpdateRequest serviceGoodSingleUpdateRequest) {
        if (serviceGoodSingleUpdateRequest == null || serviceGoodSingleUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 更新服务类商品的基本信息
        Good good = new Good();
        BeanUtils.copyProperties(serviceGoodSingleUpdateRequest, good);
        good.setType("服务类");
        good.setIsGoodType(0);
        good.setInventory(1);
        good.setFestivalName("无");
        // 校验
        goodService.validGood(good, true);
        boolean result = goodService.updateById(good);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "服务类商品信息更新失败");

        return ResultUtils.success(true);
    }





}
