package com.cultural.heritage.controller.order;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.OrderStatusConstant;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.cart.CartOrderAddRequest;
import com.cultural.heritage.model.dto.cart.CartOrderItemAddRequest;
import com.cultural.heritage.model.dto.order.OrderQueryRequest;
import com.cultural.heritage.model.dto.order.OrderUpdateRequest;
import com.cultural.heritage.model.dto.order.capital.OrderItemMainInfoAddRequest;
import com.cultural.heritage.model.dto.order.capital.OrderMainInfoAddRequest;
import com.cultural.heritage.model.dto.snapshot.GoodSnapshot;
import com.cultural.heritage.model.entity.*;
import com.cultural.heritage.model.vo.order.OrderVO;
import com.cultural.heritage.service.good.CartRecordService;
import com.cultural.heritage.service.good.GoodService;
import com.cultural.heritage.service.order.OrderItemService;
import com.cultural.heritage.service.order.OrderService;
import com.cultural.heritage.service.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/order")
@Slf4j
@Tag(name = "订单管理模块")
@Transactional(rollbackFor = Exception.class)
public class OrderController {

    @Resource
    private OrderService orderService;



    @Resource
    private OrderItemService orderItemService;



    @Resource
    private UserService userService;



    @Resource
    private CartRecordService cartRecordService;


    @Resource
    private GoodService goodService;



    /**
     * 用户通过购物车创建订单
     */
    @PostMapping("/add/cart")
    @Transactional(rollbackFor = Exception.class)
    @Operation(summary = "用户通过购物车创建订单", description = "参数：购物车订单创建请求体，权限：所有人，方法名：addCartOrder")
    public BaseResponse<Long> addCartOrder(@RequestBody CartOrderAddRequest cartOrderAddRequest, HttpServletRequest request) {
        if (cartOrderAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 校验用户当前是否处于登录态
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();


        // 封装成订单创建请求体
        OrderMainInfoAddRequest orderMainInfoAddRequest = new OrderMainInfoAddRequest();
        BeanUtils.copyProperties(cartOrderAddRequest, orderMainInfoAddRequest);

        // 校验购物车数据是否准确
        List<CartOrderItemAddRequest> cartOrderItemAddRequestList = cartOrderAddRequest.getCartOrderItemAddRequestList();
        boolean isAccurate = cartRecordService.validIsConsistent(cartOrderItemAddRequestList);
        ThrowUtils.throwIf(!isAccurate, ErrorCode.SYSTEM_ERROR, "当前购物车中的某些商品已下架或者库存不足，无法生成订单");

        // 获取购物车id列表
        List<Long> cartIds = cartOrderItemAddRequestList.stream().map(CartOrderItemAddRequest::getCartRecordId).toList();
        // 根据id列表获取购物车商品信息
        QueryWrapper<CartRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", cartIds);
        List<CartRecord> cartRecordList = cartRecordService.list(queryWrapper);

        // 计算订单总金额
        BigDecimal totalAmount = BigDecimal.ZERO;
        for (CartRecord cartRecord : cartRecordList) {
            totalAmount = totalAmount.add(cartRecord.getSubtotal());
        }
        orderMainInfoAddRequest.setTotalAmount(totalAmount);

        // 封装成一个map集合（键：购物车id, 值：购物车商品数量）
        Map<Long, Integer> map = new HashMap<>();
        for (CartOrderItemAddRequest cartOrderItemAddRequest : cartOrderItemAddRequestList) {
            Long cartRecordId = cartOrderItemAddRequest.getCartRecordId();
            Integer quantity = cartOrderItemAddRequest.getQuantity();
            map.put(cartRecordId, quantity);
        }
        // 封装成订单明细主要信息请求体列表
        List<OrderItemMainInfoAddRequest> orderItemMainInfoAddRequestList = new ArrayList<>();
        OrderItemMainInfoAddRequest orderItemMainInfoAddRequest;
        for (CartRecord cartRecord : cartRecordList) {
            Long goodId = cartRecord.getGoodId();
            Long id = cartRecord.getId();
            Integer quantity = map.get(id);
            orderItemMainInfoAddRequest = new OrderItemMainInfoAddRequest(goodId, quantity, null, null);
            orderItemMainInfoAddRequestList.add(orderItemMainInfoAddRequest);
        }
        // 填充到订单主要信息请求体
        orderMainInfoAddRequest.setOrderItemMainInfoAddRequestList(orderItemMainInfoAddRequestList);
        // 创建通用订单(常规类和服务类商品)
        Long orderId = orderService.createCommonOrder(orderMainInfoAddRequest, userId, true, cartIds);
        return ResultUtils.success(orderId);
    }



    /**
     * 小程序端创建用户订单
     * @param orderMainInfoAddRequest 订单创建请求体
     * @return  是否创建成功
     */
    @PostMapping("/add")
    @Transactional(rollbackFor = Exception.class)
    @Operation(summary = "小程序端创建用户订单", description = "参数：订单创建请求体，权限：所有人，方法名：addOrder")
    public BaseResponse<Long> addOrder(@RequestBody OrderMainInfoAddRequest orderMainInfoAddRequest, HttpServletRequest request) {
        if (orderMainInfoAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 校验用户当前是否处于登录态
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        // 校验单独购买的商品数据是否准确
        orderService.validSingleGoodOrder(orderMainInfoAddRequest);
        // 创建通用订单(常规类和服务类商品)
        Long orderId = orderService.createCommonOrder(orderMainInfoAddRequest, userId, false, null);
        return ResultUtils.success(orderId);
    }



    /**
     * 分页查询所有订单(包括订单明细)
     * @param orderQueryRequest  订单查询请求体
     * @return 订单列表信息
     */
    @PostMapping("/list/page")
    @Operation(summary = "Web端管理员分页查询订单", description = "参数：订单查询请求体，权限：管理员(admin, boss)，方法名：listOrder")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<OrderVO>> listOrder(@RequestBody OrderQueryRequest orderQueryRequest) {
        if (orderQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = orderQueryRequest.getCurrent();
        long pageSize = orderQueryRequest.getPageSize();
        Page<Order> page = orderService.page(new Page<>(current, pageSize),
                orderService.getQueryWrapper(orderQueryRequest));
        List<Order> orders = page.getRecords();
        List<OrderVO> orderVOS = orders.stream().map(order -> {
            OrderVO orderVO = new OrderVO();
            BeanUtils.copyProperties(order, orderVO);
            return orderVO;
        }).toList();
        orderVOS.forEach(orderVO -> {
            Long id = orderVO.getId();
            QueryWrapper<OrderItems> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("orderId", id);
            List<OrderItems> list = orderItemService.list(queryWrapper);
            orderVO.setOrderItemList(list);
        });
        Page<OrderVO> orderVOPage = new Page<>();
        orderVOPage.setRecords(orderVOS);
        orderVOPage.setTotal(page.getTotal());
        orderVOPage.setPages(page.getPages());
        orderVOPage.setCurrent(page.getCurrent());
        orderVOPage.setSize(page.getSize());
        return ResultUtils.success(orderVOPage);
    }



    /**
     * 小程序端用户查询订单
     * @return  用户订单列表
     */
    @PostMapping("/list")
    @Operation(summary = "小程序端用户查询订单", description = "参数：用户id请求体，权限：所有人，方法名：listUserOrder")
    public BaseResponse<List<OrderVO>> listUserOrder(HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long id = loginUser.getId();
        QueryWrapper<Order> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", id);
        List<Order> orders = orderService.list(queryWrapper);
        List<OrderVO> orderVOS = orders.stream().map(order -> {
            Long orderId = order.getId();
            QueryWrapper<OrderItems> orderItemsQueryWrapper = new QueryWrapper<>();
            orderItemsQueryWrapper.eq("orderId", orderId);
            List<OrderItems> orderItemsList = orderItemService.list(orderItemsQueryWrapper);
            OrderVO orderVO = new OrderVO();
            BeanUtils.copyProperties(order, orderVO);
            orderVO.setOrderItemList(orderItemsList);
            return orderVO;
        }).toList();
        return ResultUtils.success(orderVOS);
    }


    /**
     * Web端管理员更新订单状态
     * @param orderUpdateRequest  订单状态更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "Web端管理员更新订单状态", description = "参数：订单状态更新请求体，权限：管理员(admin, boss)，方法名：updateOrderStatus")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateOrderStatus(@RequestBody OrderUpdateRequest orderUpdateRequest) {
        if (orderUpdateRequest == null || orderUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = orderUpdateRequest.getId();
        String orderStatus = orderUpdateRequest.getOrderStatus();
        UpdateWrapper<Order> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("orderStatus", orderStatus);
        boolean result = orderService.update(updateWrapper);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }


    /**
     * 小程序端根据订单id查询订单信息
     * @param commonRequest     根据id查询请求体
     * @return   用户订单信息
     */
    @PostMapping ("/get/id")
    @Operation(summary = "小程序端用户根据id查询订单", description = "参数：根据id查询请求体，权限：所有人，方法名：getOrderById")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<OrderVO> getOrderById(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        Long orderId = commonRequest.getId();
        Order order = orderService.getById(orderId);
        ThrowUtils.throwIf(order == null, ErrorCode.NOT_FOUND_ERROR, "订单不存在");
        OrderVO orderVO = new OrderVO();
        BeanUtils.copyProperties(order, orderVO);
        QueryWrapper<OrderItems> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("orderId", orderId);
        List<OrderItems> orderItemsList = orderItemService.list(queryWrapper);
        orderVO.setOrderItemList(orderItemsList);
        return ResultUtils.success(orderVO);
    }



    /**
     * 小程序端用户取消订单
     * @param commonRequest     订单id
     * @return   是否取消成功
     */
    @PostMapping ("/cancel/id")
    @Operation(summary = "小程序端用户取消订单", description = "参数：订单id，权限：所有人，方法名：cancelOrderById")
    public BaseResponse<Boolean> cancelOrderById(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        userService.getLoginUser(request);
        Order order = orderService.getById(id);
        ThrowUtils.throwIf(order == null, ErrorCode.OPERATION_ERROR, "订单不存在");
        order.setOrderStatus(OrderStatusConstant.TRANSACTION_CLOSED);
        boolean update = orderService.updateById(order);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "订单状态更新失败");

        // 恢复商品库存
        QueryWrapper<OrderItems> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("orderId", id);
        List<OrderItems> orderItemsList = orderItemService.list(queryWrapper);
        // 获取商品id列表
        List<Long> goodIds = orderItemsList.stream().map(orderItems -> {
            GoodSnapshot goodSnapshot = orderItems.getGoodSnapshot();
            return goodSnapshot.getId();
        }).toList();
        QueryWrapper<Good> goodQueryWrapper = new QueryWrapper<>();
        goodQueryWrapper.in("id", goodIds);
        // 获取商品列表
        List<Good> goodList = goodService.list(goodQueryWrapper);
        // 封装为map集合（键：商品id, 值：库存量）,用来存储购买的商品id对应的购买数量
        Map<Long, Integer> map = new HashMap<>();
        for (OrderItems orderItems : orderItemsList) {
            GoodSnapshot goodSnapshot = orderItems.getGoodSnapshot();
            Long goodId = goodSnapshot.getId();
            Integer quantity = orderItems.getQuantity();
            map.put(goodId, quantity);
        }

        // 批量更新商品库存
        for (Good good : goodList) {
            Long goodId = good.getId();
            // 已经下单购买的数量
            Integer quantity = map.get(goodId);
            // 原有的库存
            Integer inventory = good.getInventory();
            good.setInventory(quantity + inventory);
        }
        boolean updateBatch = goodService.updateBatchById(goodList);
        ThrowUtils.throwIf(!updateBatch, ErrorCode.SYSTEM_ERROR, "商品库存恢复失败");
        return ResultUtils.success(true);
    }


}
