package com.cultural.heritage.service.order.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.constant.CommonConstant;
import com.cultural.heritage.constant.OrderStatusConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.mapper.*;
import com.cultural.heritage.model.dto.order.OrderAddRequest;
import com.cultural.heritage.model.dto.order.OrderItemAddRequest;
import com.cultural.heritage.model.dto.order.OrderQueryRequest;
import com.cultural.heritage.model.dto.order.capital.OrderItemMainInfoAddRequest;
import com.cultural.heritage.model.dto.order.capital.OrderMainInfoAddRequest;
import com.cultural.heritage.model.dto.snapshot.AddressSnapshot;
import com.cultural.heritage.model.dto.snapshot.ContactsSnapshot;
import com.cultural.heritage.model.dto.snapshot.CouponSnapshot;
import com.cultural.heritage.model.dto.snapshot.GoodSnapshot;
import com.cultural.heritage.model.entity.Good;
import com.cultural.heritage.model.entity.Order;
import com.cultural.heritage.model.entity.OrderItems;
import com.cultural.heritage.model.enums.GoodTypeEnum;
import com.cultural.heritage.service.good.CartRecordService;
import com.cultural.heritage.service.good.GoodService;
import com.cultural.heritage.service.order.OrderItemService;
import com.cultural.heritage.service.order.OrderService;
import com.cultural.heritage.utils.OrderNumberUtils;
import com.cultural.heritage.utils.SqlUtils;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements OrderService {


    @Resource
    private AddressMapper addressMapper;


    @Resource
    private ContactsMapper contactsMapper;


    @Resource
    private CouponMapper couponMapper;


    @Resource
    private GoodMapper goodMapper;


    @Resource
    private GoodService goodService;


    @Resource
    private OrderItemService orderItemService;


    @Resource
    private CartRecordService cartRecordService;


    /**
     * 获取查询条件
     */
    @Override
    public QueryWrapper<Order> getQueryWrapper(OrderQueryRequest orderQueryRequest) {
        if (orderQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long userId = orderQueryRequest.getUserId();
        Long orderId = orderQueryRequest.getOrderId();
        BigDecimal minTotalAmount = orderQueryRequest.getMinTotalAmount();
        BigDecimal maxTotalAmount = orderQueryRequest.getMaxTotalAmount();
        String orderStatus = orderQueryRequest.getOrderStatus();
        String orderNumber = orderQueryRequest.getOrderNumber();
        Date startTime = orderQueryRequest.getStartTime();
        Date endTime = orderQueryRequest.getEndTime();
        String sortOrder = orderQueryRequest.getSortOrder();
        String sortField = orderQueryRequest.getSortField();
        String orderType = orderQueryRequest.getOrderType();

        QueryWrapper<Order> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ObjectUtils.isNotEmpty(userId), "userId", userId);
        queryWrapper.eq(ObjectUtils.isNotEmpty(orderId), "id", orderId);
        queryWrapper.ge(ObjectUtils.isNotEmpty(minTotalAmount), "totalAmount", minTotalAmount);
        queryWrapper.le(ObjectUtils.isNotEmpty(maxTotalAmount), "totalAmount", maxTotalAmount);
        queryWrapper.eq(StringUtils.isNotBlank(orderStatus), "orderStatus", orderStatus);
        queryWrapper.eq(StringUtils.isNotBlank(orderType), "orderType", orderType);
        queryWrapper.between(startTime != null && endTime != null, "createTime", startTime, endTime);
        queryWrapper.eq(StringUtils.isNotBlank(orderNumber), "orderNumber", orderNumber);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);
        return queryWrapper;
    }


    /**
     * 封装订单详细信息请求体
     */
    @Override
    public OrderAddRequest initOrderMainInfo(OrderMainInfoAddRequest orderMainInfoAddRequest) {
        String userName = orderMainInfoAddRequest.getUserName();
        Long addressId = orderMainInfoAddRequest.getAddressId();
        Long contactsId = orderMainInfoAddRequest.getContactsId();
        Long couponId = orderMainInfoAddRequest.getCouponId();
        String orderType = orderMainInfoAddRequest.getOrderType();
        String note = orderMainInfoAddRequest.getNote();
        BigDecimal totalAmount = orderMainInfoAddRequest.getTotalAmount();
        List<OrderItemMainInfoAddRequest> orderItemMainInfoAddRequestList = orderMainInfoAddRequest.getOrderItemMainInfoAddRequestList();


        GoodTypeEnum orderTypeEnum = GoodTypeEnum.getEnumByValue(orderType);
        ThrowUtils.throwIf(orderTypeEnum == null, ErrorCode.PARAMS_ERROR);
        AddressSnapshot addressSnapshot = null;
        ContactsSnapshot contactsSnapshot = null;
        CouponSnapshot couponSnapshot = null;

        // 获取订单地址信息
        if (orderTypeEnum.equals(GoodTypeEnum.PRODUCT)) {
            addressSnapshot = getSnapshot(addressId, addressMapper, AddressSnapshot.class);
        }
        // 获取订单联系人信息
        if (orderTypeEnum.equals(GoodTypeEnum.SERVICE)) {
            contactsSnapshot = getSnapshot(contactsId, contactsMapper, ContactsSnapshot.class);
        }
        // 获取订单优惠券信息
        if (couponId != null) {
            couponSnapshot = getSnapshot(couponId, couponMapper, CouponSnapshot.class);
        }
        // 封装订单明细信息
        List<OrderItemAddRequest> orderItemAddRequestList = orderItemMainInfoAddRequestList.stream().map(orderItemMainInfoAddRequest -> {
            Long goodId = orderItemMainInfoAddRequest.getGoodId();
            Integer quantity = orderItemMainInfoAddRequest.getQuantity();
            String reservationDate = orderItemMainInfoAddRequest.getReservationDate();
            String timeSlot = orderItemMainInfoAddRequest.getTimeSlot();
            OrderItemAddRequest orderItemAddRequest = new OrderItemAddRequest();
            // 获取订单商品信息
            GoodSnapshot goodSnapshot = getSnapshot(goodId, goodMapper, GoodSnapshot.class);

            orderItemAddRequest.setGoodSnapshot(goodSnapshot);
            orderItemAddRequest.setPriceSnapshot(goodSnapshot.getPrice());
            orderItemAddRequest.setQuantity(quantity);
            orderItemAddRequest.setItemTotalAmount(goodSnapshot.getPrice().multiply(BigDecimal.valueOf(quantity)));
            orderItemAddRequest.setReservationDate(reservationDate);
            orderItemAddRequest.setTimeSlot(timeSlot);
            return orderItemAddRequest;
        }).toList();

        // 创建订单详细信息请求体
        OrderAddRequest orderAddRequest = new OrderAddRequest();
        orderAddRequest.setUserName(userName);
        orderAddRequest.setOrderType(orderType);
        orderAddRequest.setNote(note);
        orderAddRequest.setTotalAmount(totalAmount);
        orderAddRequest.setAddressSnapshot(addressSnapshot);
        orderAddRequest.setContactsSnapshot(contactsSnapshot);
        orderAddRequest.setCouponSnapshot(couponSnapshot);
        orderAddRequest.setOrderItemList(orderItemAddRequestList);
        return orderAddRequest;
    }



    /**
     * 校验订单
     */
    @Override
    public void validOrder(Order order) {
        String orderType = order.getOrderType();
        String orderNumber = order.getOrderNumber();
        Long userId = order.getUserId();
        String userName = order.getUserName();
        BigDecimal totalAmount = order.getTotalAmount();
        String orderStatus = order.getOrderStatus();
        if (ObjectUtils.isEmpty(totalAmount) || totalAmount.compareTo(BigDecimal.ZERO) <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "价格错误");
        }
        if (ObjectUtils.isEmpty(userId)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        if (StringUtils.isAnyBlank(orderType, orderNumber, userName, orderStatus)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "参数不全");
        }
    }




    /**
     * 创建通用订单(常规类和服务类商品)
     */
    @Override
    public Long createCommonOrder(OrderMainInfoAddRequest orderMainInfoAddRequest, Long userId, boolean isCartOrder, List<Long> cartIds) {
        // 封装订单详细信息请求体
        OrderAddRequest orderAddRequest = this.initOrderMainInfo(orderMainInfoAddRequest);
        if (orderAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 创建一个总订单
        Order order = new Order();
        BeanUtils.copyProperties(orderAddRequest, order);
        order.setUserId(userId);
        // 生成订单编号
        String orderNumber = OrderNumberUtils.generateOrderId();
        order.setOrderNumber(orderNumber);
        // 设置物流单号
        order.setTrackingNumber("无");
        // 设置订单状态
        order.setOrderStatus(OrderStatusConstant.PENDING_PAYMENT);
        // 校验订单
        this.validOrder(order);
        boolean result = this.save(order);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        // 创建订单明细
        Long id = order.getId();
        List<OrderItemAddRequest> orderItemList = orderAddRequest.getOrderItemList();
        List<OrderItems> newOrderItemsList = orderItemList.stream().map(orderItem -> {
            OrderItems orderItems = new OrderItems();
            BeanUtils.copyProperties(orderItem, orderItems);
            orderItems.setOrderId(id);
            return orderItems;
        }).toList();
        boolean save = orderItemService.saveBatch(newOrderItemsList);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR);

        // 扣减商品库存
        List<OrderItemMainInfoAddRequest> orderItemMainInfoAddRequestList = orderMainInfoAddRequest.getOrderItemMainInfoAddRequestList();
        // 封装map集合（键：商品id, 值：购买数量）
        Map<Long, Integer> map = new HashMap<>();
        for (OrderItemMainInfoAddRequest orderItemMainInfoAddRequest : orderItemMainInfoAddRequestList) {
            Long goodId = orderItemMainInfoAddRequest.getGoodId();
            Integer quantity = orderItemMainInfoAddRequest.getQuantity();
            map.put(goodId, quantity);
        }
        // 批量修改商品库存信息
        List<Long> goodIds = orderItemMainInfoAddRequestList.stream().map(OrderItemMainInfoAddRequest::getGoodId).toList();
        QueryWrapper<Good> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", goodIds);
        List<Good> goodList = goodService.list(queryWrapper);
        for (Good good : goodList) {
            Integer inventory = map.get(good.getId());
            good.setInventory(good.getInventory() - inventory);
        }
        boolean update = goodService.updateBatchById(goodList);
        ThrowUtils.throwIf(!update, ErrorCode.SYSTEM_ERROR, "商品库存更新失败");

        // 清空购物车
        if (isCartOrder) {
            boolean remove = cartRecordService.removeBatchByIds(cartIds);
            ThrowUtils.throwIf(!remove, ErrorCode.SYSTEM_ERROR, "清空购物车失败");
        }
        return id;
    }



    /**
     * 校验单独购买的商品数据是否准确
     */
    @Override
    public void validSingleGoodOrder(OrderMainInfoAddRequest orderMainInfoAddRequest) {
        List<OrderItemMainInfoAddRequest> orderItemMainInfoAddRequestList = orderMainInfoAddRequest.getOrderItemMainInfoAddRequestList();
        if (orderItemMainInfoAddRequestList.size() != 1) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "购买了多种商品或未购买商品");
        }
        OrderItemMainInfoAddRequest orderItemMainInfoAddRequest = orderItemMainInfoAddRequestList.get(0);
        Long goodId = orderItemMainInfoAddRequest.getGoodId();
        Integer quantity = orderItemMainInfoAddRequest.getQuantity();
        Good good = goodService.getById(goodId);
        ThrowUtils.throwIf(good == null || good.getIsShelves() == 0, ErrorCode.SYSTEM_ERROR, "商品已下架或者不存在");
        ThrowUtils.throwIf(quantity > good.getInventory(), ErrorCode.SYSTEM_ERROR, "商品库存不足");
    }




    /**
     * 根据id获取订单(地址，联系人，优惠券，商品)信息
     */
    private static <T, S> S getSnapshot(Long id, BaseMapper<T> mapper, Class<S> snapshotClass) {
        // 创建查询条件
        QueryWrapper<T> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", id);
        // 查询实体对象
        T entity = mapper.selectOne(queryWrapper);
        // 判断实体对象是否存在，不存在则抛出异常
        ThrowUtils.throwIf(entity == null, ErrorCode.NOT_FOUND_ERROR);
        try {
            // 创建快照对象实例
            S snapshot = snapshotClass.getDeclaredConstructor().newInstance();
            // 复制属性到快照对象
            BeanUtils.copyProperties(entity, snapshot);
            return snapshot;
        } catch (Exception e) {
            throw new BusinessException(ErrorCode.SYSTEM_ERROR);
        }
    }



}
